'''Information about supported locales, countries, and languages.'''

# this is part of langpack-o-matic, by Martin Pitt <martin.pitt@canonical.com>
# (C) 2005 Canonical Ltd.

import os.path, tempfile, shutil, tarfile

class SupportedLocales:
    def __init__(self):
        self.languages = {}
        mapdir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 
            '..', 'maps')
        for line in open(os.path.join(mapdir, 'languages')):
            if line.find('#') == -1:
                (code, name) = line.strip().split(':')
                self.languages[code] = name

        self.countries = {}
        for line in open(os.path.join(mapdir, 'countries')):
            if line.find('#') == -1 and line.find(':') >= 0:
                (code, name) = line.strip().split(':')
                self.countries[code] = name

        self.locales = {}
        for s in open(os.path.join(mapdir, 'supported-locales')):
            (l, e) = s.split()
            self.locales[l] = e

        self.pkglocales = {}
        for s in open(os.path.join(mapdir, 'locale2pkgname')):
            (l, p) = s.strip().split(':')
            (ll, cc) = l.split('-')
            loc = "%s_%s" % (ll, cc.upper())
            if not self.pkglocales.has_key(p):
                self.pkglocales[p] = []
            self.pkglocales[p].append(loc)

    def known_language(self, lang):
        '''Return whether the given language is supported.

        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_language('en')
        True
        >>> sl.known_language('de')
        True
        >>> sl.known_language('xx')
        False
        '''

        return self.languages.has_key(lang)

    def known_country(self, country):
        '''Return whether the given language is supported.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_country('GB')
        True
        >>> sl.known_country('US')
        True
        >>> sl.known_country('DE')
        True
        >>> sl.known_country('EN')
        False
        '''

        return self.countries.has_key(country)

    def known_locale(self, locale):
        '''Return whether the given locale is supported.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.known_locale('en_GB.UTF-8')
        True
        >>> sl.known_locale('de_DE.UTF-8')
        True
        >>> sl.known_locale('de_DE@euro')
        False
        >>> sl.known_locale('de_DE@euro.UTF-8')
        False
        >>> sl.known_locale('xx_XX')
        False
        '''

        return self.locales.has_key(locale)

    def language_name(self, code):
        '''Return the name of the given language code.
        
        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.language_name('de')
        'German'
        >>> sl.language_name('en')
        'English'
        >>> sl.language_name('xx')
        Traceback (most recent call last):
        KeyError: 'xx'
        '''

        return self.languages[code]

    def country_name(self, code):
        '''Return the name of the given country code.

        >>> import localeinfo
        >>> sl = localeinfo.SupportedLocales()
        >>> sl.country_name('GB')
        'United Kingdom'
        >>> sl.country_name('US')
        'United States'
        >>> sl.country_name('DE')
        'Germany'
        >>> sl.country_name('EN')
        Traceback (most recent call last):
        KeyError: 'EN'
        '''

        return self.countries[code]

    def language_locales(self, lang):
        '''Return a list with pairs (locale, encoding) that are supported for
        the given language code.  

        Return empty list if language is unknown.
        
        >>> import localeinfo
        >>> sl=localeinfo.SupportedLocales()
        >>>
        >>> sl.language_locales('foo')
        []
        >>> sl.language_locales('ce')
        Error: no supported locales for known language ce
        []
        >>> sl.language_locales('cs') == [('cs_CZ.UTF-8', 'UTF-8')]
        True
        >>> sl.language_locales('eo')
        [('eo_US.UTF-8', 'UTF-8'), ('eo.UTF-8', 'UTF-8')]
        >>> sl.language_locales('zh-hans')
        [('zh_CN.UTF-8', 'UTF-8'), ('zh_SG.UTF-8', 'UTF-8')]
        >>> sl.language_locales('zh-hant')
        [('zh_HK.UTF-8', 'UTF-8'), ('zh_TW.UTF-8', 'UTF-8')]
        '''

        if self.pkglocales.has_key(lang):
            locales = [(l, e) for l, e in self.locales.iteritems() if l.split('.')[0].split('@')[0] in self.pkglocales[lang]]
        else:
            locales = [(l, e) for l, e in self.locales.iteritems() if lang == l.split('.')[0].split('@')[0].split('_')[0]]

        # maps consistency check
        if len(locales) == 0 and self.known_language(lang):
            print 'Error: no supported locales for known language', lang

        return locales

    def create_locale_tar(self, lang):
        '''Generate a tarball (suitable for supplying as extra.tar) with 
        the /var/lib/locales/supported.d file.
        
        The returned tarball name is a temporary file that should be removed
        after using it.
        
        >>> import localeinfo, os, tarfile
        >>> sl = localeinfo.SupportedLocales()
        >>> tarname = sl.create_locale_tar('de')

        >>> tar = tarfile.open(tarname)
        >>> names = tar.getnames()
        >>> names.sort()
        >>> names
        ['var', 'var/lib', 'var/lib/locales', 'var/lib/locales/supported.d', 'var/lib/locales/supported.d/de']
        >>> print tar.extractfile('var/lib/locales/supported.d/de').read()
        de_DE.UTF-8 UTF-8
        de_CH.UTF-8 UTF-8
        de_BE.UTF-8 UTF-8
        de_LI.UTF-8 UTF-8
        de_LU.UTF-8 UTF-8
        de_AT.UTF-8 UTF-8
        <BLANKLINE>
        >>> tar.close()
        >>> os.unlink(tarname)
        '''

        dir = None
        tar = None
        try:
            dir = tempfile.mkdtemp()
            supported_dir = os.path.join(dir, 'var/lib/locales/supported.d')
            os.makedirs(supported_dir)
            supp_list = open(os.path.join(supported_dir, lang), 'w')

            sl = self.language_locales(lang)

            for loc, enc in sl:
                print >> supp_list, '%s %s' % (loc, enc)

            supp_list.close()
            (f, tar) = tempfile.mkstemp('.tar.gz')
            os.close(f)

            t = tarfile.open(tar, 'w:gz')
            old_cwd = os.getcwd()
            os.chdir(dir)
            t.add('.')
            os.chdir(old_cwd)
            t.close()
        finally:
            shutil.rmtree(dir, True)

        return tar

# unit test
if __name__ == '__main__':
    import doctest
    doctest.testmod()
